@@ -39,7 +39,7 @@ return.
 
 =head3 css
 
-(not actually a template, it's the default CSS).
+The default CSS, with some minor templating.
 
 =head3 bare_html
 
@@ -51,6 +51,10 @@ Muse headers).
 Minimal (but valid) XHTML template, with a link to C<stylesheet.css>.
 Meant to be used in the EPUB generation.
 
+=head3 title_page_html
+
+Minimal XHTML template for a title page in the EPUB generation.
+
 =head3 bare_latex
 
 Minimal and uncomplete LaTeX chunck, meant to be used when merging
@@ -345,8 +349,11 @@ sub css {
     if (my $ref = $self->ttref('css')) {
         return $ref;
     }
-    # not a template, just a static file
-        my $css = <<'EOF';
+    my $css = <<'EOF';
+[% IF epub %]
+@page { margin: 5pt; }
+[% END %]
+
 html,body {
 	margin:0;
 	padding:0;
@@ -355,10 +362,33 @@ html,body {
 	font-family: serif;
 	font-size: 10pt;
 }
+
+[% IF epub %]
+div#page > p {
+   margin: 0;
+   text-indent: 1em;
+   text-align: justify;
+}
+
+blockquote > p, li > p {
+   margin-top: 5pt;
+   text-indent: 0em;
+   text-align: justify;
+}
+
+a {
+   color:#000000;
+   text-decoration: underline
+}
+[%  END %]
+
+[% IF html %]
 div#page {
    margin:20px;
    padding:20px;
 }
+[% END %]
+
 pre, code {
     font-family: Consolas, courier, monospace;
 }
@@ -483,33 +513,27 @@ div.right {
     text-align: right;
 }
 
-div#tableofcontents{
-    padding:20px;
-}
-
-#tableofcontents p {
-    margin: 3px 1em;
-    text-indent: -1em;
-}
-
 .toclevel1 {
 	font-weight: bold;
-	font-size:11pt
+	font-size:11pt;
 }	
 
 .toclevel2 {
 	font-weight: bold;
 	font-size: 10pt;
+    padding-left: 1em;
 }
 
 .toclevel3 {
 	font-weight: normal;
 	font-size: 9pt;
+    padding-left: 2em;
 }
 
 .toclevel4 {
 	font-weight: normal;
 	font-size: 8pt;
+    padding-left: 3em;
 }
 
 
@@ -560,6 +584,41 @@ EOF
     return \$html;
 }
 
+sub title_page_html {
+    my $self = shift;
+    if (my $ref = $self->ttref('title_page_html')) {
+        return $ref;
+    }
+    my $html = <<'EOF';
+<div id="first-page-title-page">
+  [% IF doc.header_defined.author %]
+  <h2 class="amw-text-author">[% doc.header_as_html.author %]</h2>
+  [% END %]
+  <h1 class="amw-text-title">[% doc.header_as_html.title %]</h1>
+  [% IF doc.header_defined.subtitle %]
+  <h2 class="amw-text-subtitle">[% doc.header_as_html.subtitle %]</h2>
+  [% END  %]
+  [% IF doc.header_defined.date %]
+  <h3 class="amw-text-date">[% doc.header_as_html.date %]</h3>
+  [% END  %]
+</div>
+<hr />
+<div id="impressum-title-page">
+  [% IF doc.header_defined.source %]
+  <div class="amw-text-source" id="source">
+  [% doc.header_as_html.source %]
+  </div>
+  [% END %]
+  [% IF doc.header_defined.notes %]
+  <div class="amw-text-notes" id="notes">
+  [% doc.header_as_html.notes %]
+  </div>
+  [% END %]
+</div>
+EOF
+    return \$html;
+}
+
 sub minimal_html {
     my $self = shift;
     if (my $ref = $self->ttref('minimal_html')) {
